;;;-*- Mode: Lisp; Package: CCL -*-
;;;
;;;   Copyright (C) 1994-2001 Digitool, Inc
;;;   This file is part of Opensourced MCL.
;;;
;;;   Opensourced MCL is free software; you can redistribute it and/or
;;;   modify it under the terms of the GNU Lesser General Public
;;;   License as published by the Free Software Foundation; either
;;;   version 2.1 of the License, or (at your option) any later version.
;;;
;;;   Opensourced MCL is distributed in the hope that it will be useful,
;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;   Lesser General Public License for more details.
;;;
;;;   You should have received a copy of the GNU Lesser General Public
;;;   License along with this library; if not, write to the Free Software
;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;;;

(eval-when (:compile-toplevel :execute)
  (require "LINUX-SYSCALLS"))

(defun current-directory-name ()
  (flet ((try-getting-dirname (bufsize)
	   (%stack-block ((buf bufsize))
	     (let* ((len (syscall linux::getcwd buf bufsize)))
	       (cond ((< len 0) (%errno-disp len))
		     ((< len bufsize)
		      (setf (%get-unsigned-byte buf len) 0)
		      (values (%get-cstring buf) len))
		     (t (values nil len)))))))
    (do* ((string nil)
	  (len 64)
	  (bufsize len len))
	 ((multiple-value-setq (string len) (try-getting-dirname bufsize))
	  string))))


(defun %chdir (dirname)
  (with-cstrs ((dirname dirname))
    (syscall os::chdir dirname)))

(defun %mkdir (name mode)
  (with-cstrs ((name name))
    (syscall linux::mkdir name mode)))

(defun getenv (key)
  (with-cstrs ((key (string key)))
    (let* ((env-ptr (%null-ptr)))
      (declare (dynamic-extent env-ptr))
      (%setf-macptr env-ptr (#_getenv key))
      (unless (%null-ptr-p env-ptr)
	(%get-cstring env-ptr))))
  )
;; "stat" & friends are implemented in terms of deeper, darker things that
;; need to know what version of the stat buffer they're talking about.

(defun %stat-values (result stat)
  (if (eql 0 (the fixnum result)) 
      (values
       t
       (pref stat :stat.st_mode)
       (pref stat :stat.st_size)
       (pref stat :stat.st_mtime)
       (pref stat :stat.st_ino)
       (pref stat :stat.st_uid))
      (values nil nil nil nil nil nil)))


(defun %%stat (name stat)
  (with-cstrs ((cname name))
    (%stat-values (#_ __xstat #$_STAT_VER_LINUX cname stat) stat)))

(defun %%fstat (fd stat)
  (%stat-values (#_ __fxstat #$_STAT_VER_LINUX fd stat) stat))

(defun %%lstat (name stat)
  (with-cstrs ((cname name))
    (%stat-values (#_ __lxstat #$_STAT_VER_LINUX cname stat) stat)))


;;; Returns: (values t mode size mtime inode uid) on success,
;;;          (values nil nil nil nil nil nil) otherwise
(defun %stat (name &optional link-p)
  (rlet ((stat :stat))
    (if link-p
      (%%lstat name stat)
      (%%stat name stat))))

(defun %fstat (fd)
  (rlet ((stat :stat))
    (%%fstat fd stat)))


(defun %file-kind (mode)
  (when mode
    (let* ((kind (logand mode #$__S_IFMT)))
      (cond ((eql kind #$__S_IFDIR) :directory)
	    ((eql kind #$__S_IFREG) :file)
	    ((eql kind #$__S_IFLNK) :link)
	    ((eql kind #$__S_IFIFO) :pipe)
	    ((eql kind #$__S_IFSOCK) :socket)
	    ((eql kind #$__S_IFCHR) :tty)
	    (t :special)))))

(defun %unix-file-kind (path &optional check-for-link)
  (%file-kind (nth-value 1 (%stat path check-for-link))))

(defun %unix-fd-kind (fd)
  (%file-kind (nth-value 1 (%fstat fd))))

(defun %uts-string (result idx buf)
  (if (eql 0 result)
    (%get-cstring (%inc-ptr buf (* 65 idx)))
    "unknown"))

(defun %uname (idx)
  (%stack-block ((buf (* 65 6)))  
    (%uts-string (syscall os::uname buf) idx buf)))

(defun fd-dup (fd)
  (syscall os::dup fd))

(defun fd-fsync (fd)
  (syscall os::fsync fd))

(defun fd-get-flags (fd)
  (syscall os::fcntl fd #$F_GETFL))

(defun fd-set-flags (fd new)
  (syscall os::fcntl fd #$F_SETFL new))

;;; This doesn't seem to exist on VxWorks.  It's a POSIX
;;; function AFAIK, so the source should be somewhere ...

(defun %realpath (namestring)
  (%stack-block ((resultbuf 1024))
    (with-cstrs ((name namestring))
      (let* ((result (#_realpath name resultbuf)))
        (declare (dynamic-extent result))
        (unless (%null-ptr-p result)
          (%get-cstring result))))))

; Return fully resolved pathname & file kind, or (values nil nil)

(defun %probe-file-x (namestring)
  (let* ((realpath (%realpath namestring)))
    (if realpath
      (values realpath (%unix-file-kind realpath))
      (values nil nil))))

(defun timeval->milliseconds (tv)
    (+ (* 1000 (pref tv :timeval.tv_sec)) (round (pref tv :timeval.tv_usec) 1000)))

(defun get-internal-real-time ()
  (rlet ((tv :timeval))
    (syscall os::gettimeofday tv (%null-ptr))
    (timeval->milliseconds tv)))

(defun %add-timevals (result a b)
    (let* ((seconds (+ (pref a :timeval.tv_sec) (pref b :timeval.tv_sec)))
           (millis (+ (pref a :timeval.tv_usec) (pref b :timeval.tv_usec))))
      (if (>= millis 1000000)
        (setq seconds (1+ seconds) millis (- millis 1000000)))
      (setf (pref result :timeval.tv_sec) seconds
            (pref result :timeval.tv_usec) millis)
      result))

(defun get-internal-run-time ()
  (rlet ((usage :rusage)
	 (total :timeval))
    (syscall os::getrusage #$RUSAGE_SELF  usage)
    (timeval->milliseconds (%add-timevals total 
					  (pref usage :rusage.ru_utime) 
					  (pref usage :rusage.ru_stime)))))

(defconstant unix-to-universal-time 2208988800)

(defun %file-write-date (namestring)
  (let* ((date (nth-value 3 (%stat namestring))))
    (if date
      (+ date unix-to-universal-time))))

(defun %file-author (namestring)
  (let* ((uid (nth-value 5 (%stat namestring))))
    (if uid
      (with-macptrs ((pw (#_getpwuid uid)))
        (unless (%null-ptr-p pw)
          (without-interrupts
           (%get-cstring (pref pw :passwd.pw_name))))))))

(defun isatty (fd)
  (= 1 (#_isatty fd)))

(defun %open-dir (namestring)
  (with-cstrs ((name namestring))
    (let* ((DIR (#_opendir name)))
      (unless (%null-ptr-p DIR)
	DIR))))

(defun close-dir (dir)
  (#_closedir DIR))

(defun %read-dir (dir)
  (let* ((res (#_readdir dir)))
    (unless (%null-ptr-p res)	     
      (%get-cstring (pref res :dirent.d_name)))))

(defun tcgetpgrp (fd)
  (#_tcgetpgrp fd))

(defun getpid ()
  (syscall linux::getpid))

(defun getuid ()
  (syscall linux::getuid))

(defun get-user-home-dir (userid)
  (with-macptrs ((pw (#_getpwuid userid)))
    (unless (%null-ptr-p pw)
      (without-interrupts
       (%get-cstring (pref pw :passwd.pw_dir))))))

(defun %delete-file (name)
  (with-cstrs ((n name))
    (syscall os::unlink n)))

(defun os-command (string)
  (with-cstrs ((s string))
    (#_system s)))

(defun %strerror (errno)
  (declare (fixnum errno))
  (if (< errno 0)
    (setq errno (- errno)))
  (with-macptrs (p)
    (%setf-macptr p (#_strerror errno))
    (if (%null-ptr-p p)
      (format nil "OS Error %d" errno)
      (%get-cstring p))))